@@ -5,7 +5,9 @@ from fastapi_utils.tasks import repeat_every
 from icecream import ic
 from ..loop import loop
 import logging
-from ..converter.model_information import ModelInformation
+from ..model_information import ModelInformation
+from http import HTTPStatus
+from fastapi.encoders import jsonable_encoder
 
 
 class ConverterNode(Node):
@@ -27,17 +29,20 @@ class ConverterNode(Node):
                     logging.error('could not check state. Is loop reachable?')
 
     async def convert_model(self, model_information: ModelInformation):
-        if model_information.model_id in self.bad_model_ids:
+        if model_information.id in self.bad_model_ids:
             logging.info(
-                f'skipping bad model model {model_information.model_id} for {model_information.context.organization}/{model_information.context.project}.')
+                f'skipping bad model model {model_information.id} for {model_information.context.organization}/{model_information.context.project}.')
             return
         try:
+            logging.info(
+                f'converting model {jsonable_encoder(model_information)}')
             await self.converter.convert(model_information)
-            await self.converter.upload_model(model_information.context, model_information.model_id)
+            logging.info('uploading model ')
+            await self.converter.upload_model(model_information.context, model_information.id)
         except Exception as e:
-            self.bad_model_ids.append(model_information.model_id)
+            self.bad_model_ids.append(model_information.id)
             logging.error(
-                f'could not convert model {model_information.model_id} for {model_information.context.organization}/{model_information.context.project}. Details: {str(e)}.')
+                f'could not convert model {model_information.id} for {model_information.context.organization}/{model_information.context.project}. Details: {str(e)}.')
 
     async def check_state(self):
         logging.debug(f'checking state: {self.status.state}')
@@ -63,6 +68,10 @@ class ConverterNode(Node):
             project_id = project['project_id']
 
             async with loop.get(f'api{project["resource"]}') as response:
+                if response.status != HTTPStatus.OK:
+                    logging.error(
+                        f'got bad response for {response.url}: {response.status}, {response.content}')
+                    continue
                 project_categories = (await response.json())['categories']
 
             path = f'api{project["resource"]}/models'
@@ -72,12 +81,20 @@ class ConverterNode(Node):
                 models = content['models']
 
                 for model in models:
-                    if model['version']:
-                        if self.converter.source_format in model['formats'] and not self.converter.target_format in model['formats']:
-                            # if self.converter.source_format in model['formats'] and project_id == 'drawingbot' and model['version'] == "6.0":
-                            model_information = ModelInformation(
-                                organization=organization_id, project=project_id, model_id=model['id'], project_categories=project_categories, version=model['version'])
-                            await self.convert_model(model_information)
+                    if (model['version']
+                            and self.converter.source_format in model['formats']
+                            and self.converter.target_format not in model['formats']
+                            ):
+                        # if self.converter.source_format in model['formats'] and project_id == 'drawingbot' and model['version'] == "6.0":
+                        model_information = ModelInformation(
+                            host=loop.base_url,
+                            organization=organization_id,
+                            project=project_id,
+                            id=model['id'],
+                            categories=project_categories,
+                            version=model['version'],
+                        )
+                        await self.convert_model(model_information)
 
     async def send_status(self):
         # NOTE not yet implemented
